"""
Base module to support exposing EMS SOAP Service methods
"""
import atexit
from importlib import import_module
from logging import getLogger
from os.path import dirname, realpath
from shutil import rmtree
import sys
from tempfile import mkdtemp

from commonconf import settings
from restclients_core.util.mock import convert_to_platform_safe
from suds.client import Client
from suds import byte_str, WebFault
from suds.cache import ObjectCache

from .mock import EMSMockData


def load_object_by_name(object_name):
    """Load an object from a module by name"""
    mod_name, attr = object_name.rsplit('.', 1)
    mod = import_module(mod_name)
    return getattr(mod, attr)


url_base = '/EMSAPI'


class EMSAPIException(Exception):
    pass


class EMSAPI(object):
    """EMS API base
    """

    def __init__(self, options={}):
        if hasattr(settings, 'EMS_API_HOST'):
            self._wsdl = '%s%s/%s' % (
                settings.EMS_API_HOST, url_base, options.get('wsdl', ''))
            self._data = self._live
        else:
            self._wsdl = 'file://%s/resources/ems/file/%s/%s' % (
                dirname(realpath(__file__)), url_base,
                convert_to_platform_safe(options.get('wsdl', '')))
            self._data = self._mock

        self._log = getLogger('ems_client')
        cache_path = mkdtemp(prefix='ems_client-', suffix='-suds')
        atexit.register(rmtree, cache_path)
        client_opts = {'cache': ObjectCache(cache_path, days=1)}
        if hasattr(settings, 'EMS_API_TRANSPORT_CLASS'):
            transport = load_object_by_name(settings.EMS_API_TRANSPORT_CLASS)
            client_opts['transport'] = transport()
            client_opts['location'] = '%s/%s/Service.asmx' % (
                settings.EMS_API_HOST, url_base)
        self._api = Client(self._wsdl, **client_opts)
        self._api.set_options(cachingpolicy=0)

    def _request(self, methodName, params={}):
        try:
            return self._data(methodName, params)
        except WebFault as err:
            self._log.exception(err)
            raise EMSAPIException(
                'Cannot connect to the EMS server: %s' % err)
        except Exception as err:
            self._log.error('Error: (%s) %s' % (err, str(sys.exc_info()[0])))

            if type(err.args[0]) is tuple and type(err.args[0][0]) is int:
                if err.args[0][0] in [401, 403]:
                    errmsg = 'The request cannot be authenticated (%s)' % (
                        err.args[0][0])
                if err.args[0][0] in [404, 405, 406, 500]:
                    errmsg = 'The server is currently unavailable (%s)' % (
                        err.args[0][0])
                else:
                    errmsg = 'Unanticipated error: %s (%s)' % (
                        err.args[0][1], err.args[0][0])
            else:
                errmsg = 'Error connecting: %s' % (err)

            raise EMSAPIException(errmsg)

    def _live(self, methodName, params={}):
        params['UserName'] = settings.EMS_API_USERNAME
        params['Password'] = settings.EMS_API_PASSWORD
        return self._api.service[self._port][methodName](**params)

    def _mock(self, methodName, params={}):
        params['__inject'] = {
            'reply': byte_str(EMSMockData().mock(
                self._port, methodName, params))
        }
        return self._api.service[self._port][methodName](**params)



from unittest import TestCase

from ems_client.service import Service


class ServiceTests(TestCase):

    def setUp(self):
        if not hasattr(self, '_api'):
            self._api = Service()

    def test_apiversion(self):
        version = self._api.get_api_version()
        self.assertGreaterEqual(version["Version"], "1.1.32.0", "API version")
        self.assertTrue(version["License"] in ["Basic", "Advanced"])

    def test_booking(self):
        booking = self._api.get_booking(8152)
        self.assertEquals(booking.event_name, 'Joe & Harold Wedding Reception')
        booking = self._api.get_booking(8806)
        self.assertEquals(booking.event_name, 'Test')

    def test_bookings(self):
        bookings = self._api.get_bookings('2016-01-01', '2016-01-01')
        self.assertGreaterEqual(len(bookings), 1)
        bookings = self._api.get_bookings('2016-01-02', '2016-01-02')
        self.assertGreaterEqual(len(bookings), 1)

    def test_buildings(self):
        buildings = self._api.get_buildings()
        self.assertGreaterEqual(len(buildings), 3)

    def test_event_types(self):
        event_types = self._api.get_event_types()
        self.assertGreaterEqual(len(event_types), 3)

    def test_rooms(self):
        rooms = self._api.get_all_rooms()
        self.assertGreaterEqual(len(rooms), 0)
        active_rooms = list(filter(lambda rm: rm.active, rooms))
        self.assertEqual(len(active_rooms), 53)

    def test_service_orders(self):
        service_orders = self._api.get_service_order_details('2016-01-01',
                                                             '2016-01-01')
        self.assertEqual(len(service_orders), 2)
        service_orders = self._api.get_service_order_details('2016-01-02',
                                                             '2016-01-02')
        self.assertEqual(len(service_orders), 4)

    def test_statuses(self):
        statuses = self._api.get_statuses()
        self.assertGreaterEqual(len(statuses), 16)
